# This file was auto-generated by Fern from our API Definition.

import typing

import httpx

from .api_keys.client import ApiKeysClient, AsyncApiKeysClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .credentials.client import AsyncCredentialsClient, CredentialsClient
from .desk.client import AsyncDeskClient, DeskClient
from .didcomm_messages.client import AsyncDidcommMessagesClient, DidcommMessagesClient
from .dids.client import AsyncDidsClient, DidsClient
from .drafts.client import AsyncDraftsClient, DraftsClient
from .environment import TruvityApiEnvironment
from .files.client import AsyncFilesClient, FilesClient
from .gdpr.client import AsyncGdprClient, GdprClient
from .keys.client import AsyncKeysClient, KeysClient
from .presentations.client import AsyncPresentationsClient, PresentationsClient


class TruvityApi:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : TruvityApiEnvironment
        The environment to use for requests from the client. from .environment import TruvityApiEnvironment



        Defaults to TruvityApiEnvironment.DEFAULT



    api_key : str
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from truvity_sdk import TruvityApi

    client = TruvityApi(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: TruvityApiEnvironment = TruvityApiEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.api_keys = ApiKeysClient(client_wrapper=self._client_wrapper)
        self.credentials = CredentialsClient(client_wrapper=self._client_wrapper)
        self.didcomm_messages = DidcommMessagesClient(client_wrapper=self._client_wrapper)
        self.dids = DidsClient(client_wrapper=self._client_wrapper)
        self.drafts = DraftsClient(client_wrapper=self._client_wrapper)
        self.files = FilesClient(client_wrapper=self._client_wrapper)
        self.keys = KeysClient(client_wrapper=self._client_wrapper)
        self.presentations = PresentationsClient(client_wrapper=self._client_wrapper)
        self.gdpr = GdprClient(client_wrapper=self._client_wrapper)
        self.desk = DeskClient(client_wrapper=self._client_wrapper)


class AsyncTruvityApi:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : TruvityApiEnvironment
        The environment to use for requests from the client. from .environment import TruvityApiEnvironment



        Defaults to TruvityApiEnvironment.DEFAULT



    api_key : str
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from truvity_sdk import AsyncTruvityApi

    client = AsyncTruvityApi(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: TruvityApiEnvironment = TruvityApiEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.api_keys = AsyncApiKeysClient(client_wrapper=self._client_wrapper)
        self.credentials = AsyncCredentialsClient(client_wrapper=self._client_wrapper)
        self.didcomm_messages = AsyncDidcommMessagesClient(client_wrapper=self._client_wrapper)
        self.dids = AsyncDidsClient(client_wrapper=self._client_wrapper)
        self.drafts = AsyncDraftsClient(client_wrapper=self._client_wrapper)
        self.files = AsyncFilesClient(client_wrapper=self._client_wrapper)
        self.keys = AsyncKeysClient(client_wrapper=self._client_wrapper)
        self.presentations = AsyncPresentationsClient(client_wrapper=self._client_wrapper)
        self.gdpr = AsyncGdprClient(client_wrapper=self._client_wrapper)
        self.desk = AsyncDeskClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: TruvityApiEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")
