# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.bad_request_error_body import BadRequestErrorBody
from ..types.problem_forbidden import ProblemForbidden
from ..types.problem_internal_server_error import ProblemInternalServerError
from ..types.problem_unauthorized import ProblemUnauthorized
from ..types.support_request_type import SupportRequestType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GdprClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def support_request_create(
        self, *, type: SupportRequestType, message: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Creates request to support.

        Parameters
        ----------
        type : SupportRequestType

        message : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from truvity_sdk import TruvityApi

        client = TruvityApi(
            api_key="YOUR_API_KEY",
        )
        client.gdpr.support_request_create(
            type="DATA_DUMP",
            message="message",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "support-requests",
            method="POST",
            json={"type": type, "message": message},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(BadRequestErrorBody, parse_obj_as(type_=BadRequestErrorBody, object_=_response.json()))  # type: ignore
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(ProblemUnauthorized, parse_obj_as(type_=ProblemUnauthorized, object_=_response.json()))  # type: ignore
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(ProblemForbidden, parse_obj_as(type_=ProblemForbidden, object_=_response.json()))  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(ProblemInternalServerError, parse_obj_as(type_=ProblemInternalServerError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncGdprClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def support_request_create(
        self, *, type: SupportRequestType, message: str, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Creates request to support.

        Parameters
        ----------
        type : SupportRequestType

        message : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from truvity_sdk import AsyncTruvityApi

        client = AsyncTruvityApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.gdpr.support_request_create(
                type="DATA_DUMP",
                message="message",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "support-requests",
            method="POST",
            json={"type": type, "message": message},
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(BadRequestErrorBody, parse_obj_as(type_=BadRequestErrorBody, object_=_response.json()))  # type: ignore
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(ProblemUnauthorized, parse_obj_as(type_=ProblemUnauthorized, object_=_response.json()))  # type: ignore
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(ProblemForbidden, parse_obj_as(type_=ProblemForbidden, object_=_response.json()))  # type: ignore
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(ProblemInternalServerError, parse_obj_as(type_=ProblemInternalServerError, object_=_response.json()))  # type: ignore
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
